#include <Servo.h>
#include <FastLED.h>

// Define connection pins:
//PIR Sensors
#define LEFT_PIR_PIN 2
#define RIGHT_PIR_PIN 5
#define BACK_PIR_PIN 8

//Servo
#define SERVO_PIN 7

//LEDS
#define LEFT_LED_PIN 3
#define RIGHT_LED_PIN 6
#define CENTER_LED_PIN 4

//Define LED colors
#define SAFECOLOR CRGB(50, 100, 150)
#define DANGERCOLOR CRGB( 255, 0, 0)
#define SENSORCOLOR CRGB( 0, 0, 100)

// FastLED CRGB object
CRGB leftLED[3];
CRGB rightLED[3];
CRGB centerLED[1];

// create servo object to control a servo
Servo myservo;
int pos = 100;    // variable to store the servo position

// Create variables for PIR readings:
int leftPIRVal = 0;
int rightPIRVal = 0;
int backPIRVal = 0;

//set time intial value and duration. This changes the length of door closing time when no threat
int timer = 0;
int timerLimit = 500;

bool motionState = false; //start with no motion detected.
bool isDoorClosed = false;//Flags door close status to show different color

void setup() {
  // Configure the pins as input or output:
  pinMode(LEFT_PIR_PIN, INPUT);
  pinMode(RIGHT_PIR_PIN, INPUT);
  pinMode(BACK_PIR_PIN, INPUT);

  pinMode(LEFT_LED_PIN, OUTPUT);
  pinMode(RIGHT_LED_PIN, OUTPUT);
  pinMode(CENTER_LED_PIN, OUTPUT);

  //setup servo and LEDs
  myservo.attach(SERVO_PIN);
  FastLED.addLeds<NEOPIXEL, LEFT_LED_PIN>(leftLED, 3);
  FastLED.addLeds<NEOPIXEL, RIGHT_LED_PIN>(rightLED, 3);
  FastLED.addLeds<NEOPIXEL, CENTER_LED_PIN>(centerLED, 1);

  //Starting Up Animation
  for (int i = 0; i < 3; i++ ) {
    leftLED[i] = SAFECOLOR;
    rightLED[i] = SAFECOLOR;

    FastLED.show();
    delay(300);
  }
  delay(300);
  centerLED[0] = SAFECOLOR;
  FastLED.show();
  delay(300);




  // Begin serial communication at a baud rate of 9600:
  Serial.begin(9600);
  delay(1000);
}


void loop() {
  // Read out the pirPin and store as val:
  leftPIRVal = digitalRead(LEFT_PIR_PIN);
  rightPIRVal = digitalRead(RIGHT_PIR_PIN);
  backPIRVal = digitalRead(BACK_PIR_PIN);

  // If motion is detected (pirPin = HIGH), do the following:
  if (leftPIRVal == HIGH || rightPIRVal == HIGH || backPIRVal == HIGH) {

    // Change the motion state to true (motion detected):
    if (motionState == false) {
      //close the door, then it is safe
      pos = 80;
      myservo.write(pos);

      //if motion is on the left
      if (leftPIRVal == HIGH) {
        Serial.println("Left Sensor Motion Detected!");

        //if door was not closed, show DANGERCOLOR
        if (isDoorClosed == false) {
          for (int i = 0; i < 3; i++ ) {
            leftLED[i] = DANGERCOLOR;
            FastLED.show();
            delay(70);
          }
        }
        //if door was closed, show SENSORCOLOR to be less alarming
        else {
          for (int i = 0; i < 3; i++ ) {
            leftLED[i] = SENSORCOLOR;
            FastLED.show();
            delay(70);
          }
        }
      }

      //if motion is on the right
      else if (rightPIRVal == HIGH) {
        Serial.println("Right Sensor Motion Detected!");
        if (isDoorClosed == false) {
          for (int i = 0; i < 3; i++ ) {
            rightLED[i] = DANGERCOLOR;
            FastLED.show();
            delay(70);
          }
        }
        else {
          for (int i = 0; i < 3; i++ ) {
            rightLED[i] = SENSORCOLOR;
            FastLED.show();
            delay(70);
          }
        }
      }

      //if motion is on the back
      else if (backPIRVal == HIGH) {
        Serial.println("Back Sensor Motion Detected!");
        if (isDoorClosed == false) {
            centerLED[0] = DANGERCOLOR;
             FastLED.show();
         delay(210);
        }
        else {
            centerLED[0] = SENSORCOLOR;
             FastLED.show();
         delay(210);
        }
        
            
      }



      //set timer and flag door state
      timer = timerLimit;
      isDoorClosed = true;
      motionState = true;

    }
  }

  // If no motion is detected (pirPins = LOW), do the following:
  else {
    // Change the motion state to false (no motion):
    if (motionState == true) {
      Serial.println("Motion ended!");

      for (int i = 0; i < 3; i++ ) {
        leftLED[i] = SAFECOLOR;
        rightLED[i] = SAFECOLOR;
        centerLED[0] = SAFECOLOR;
        FastLED.show();
        delay(300);
      }
      delay(500);
      motionState = false;

    }

    //if timer count down to 0
    if (timer == 0) {
      isDoorClosed = false;//door openning
      Serial.println("Door Open");
      //open door
      if (pos >= 0) {
        pos = pos - 20;
        myservo.write(pos);
        delay(30);
      }
    }

    //timer count down
    else {
      timer--;
      delay(10);
    }
  }
}
